;/********************************/;
;/* Demo ADC LTC1298 For ET-EXP4 */;
;/* Files Name  : LAB7-EX1.ASM   */;
;/* CPU Control : MCS51 Family   */;
;/* Assembler   : SXA51.EXE      */;
;/* Display ADC Result on LCD16X2*/;
;/********************************/;
;
PORT_IO     EQU     P0                  ; Port Interface Serial I/O
CS_EEP      EQU     PORT_IO.0           ; CS EEP = PX.0
CS_ADC      EQU     PORT_IO.1           ; CS ADC = PX.1
DO_ADC      EQU     PORT_IO.2           ; Data Output
DI_ADC      EQU     PORT_IO.3           ; Data Input
SK_ADC      EQU     PORT_IO.4           ; Clock Synchronous
;
;/* Port For Control LCD (Port-P1)
PORT_LCD    EQU     P1                  ; Port Data LCD
RS_LCD      EQU     PORT_LCD.0          ; RS Select
E_LCD       EQU     PORT_LCD.2          ; Enable LCD

            ORG     20H
CNT_BUFF:   DS      1
STACK:      DS      32

            ORG     0000H               ; Start at Monitor
START:      MOV     SP,#STACK           ; Stack = 32 Byte
            CLR     E_LCD               ; Disable LCD
            CLR     CS_EEP              ; Stop EEP
            SETB    CS_ADC              ; Stop ADC
            ;
            MOV     R2,#0FFH            ; Power-on Delay
PWR_DLY:    MOV     R3,#0FFH
            DJNZ    R3,$
            DJNZ    R2,PWR_DLY
            LCALL   INIT_LCD            ; Initial LCD4 Bit (16X2)
            ;
            MOV     A,#0                ; 1st Location Line1
            LCALL   GOTO_LCD
            LCALL   PRINT_LCD
            DB      'DEMO ADC LTC1298'  ; Message on Line-1
            DB      00H
            MOV     A,#40H              ; 1st Location Line2
            LCALL   GOTO_LCD
            LCALL   PRINT_LCD
            DB      'ADC Result :    '  ; Message on Line-2
            DB      00H
            ;
LOOP_CH0:   LCALL   READ_ADC            ; Read ADC Ch0
            MOV     A,#4DH
            LCALL   GOTO_LCD            ; Set Cursor Display
            MOV     A,DPH
            LCALL   DSP_ASCII
            MOV     A,DPL
            SWAP    A
            ANL     A,#0FH
            LCALL   DSP_ASCII
            MOV     A,DPL
            ANL     A,#0FH
            LCALL   DSP_ASCII
            SJMP    LOOP_CH0

;/************************/;
;/* Display ASCII to LCD */;
;/* Input : ACCA (HEX)   */;
;/************************/;
;
DSP_ASCII:  CJNE    A,#0AH,DSP_ASC1     ; Convert to ASCII
DSP_ASC1:   JNC     DSP_ASC2
            ADD     A,#30H              ; Convert 0-9 to ASCII
            LCALL   WR_LCD
            SJMP    DSP_ASC3
            ;
DSP_ASC2:   CLR     CY
            SUBB    A,#09H              ; Convert A-F to ASCII
            ADD     A,#40H
            LCALL   WR_LCD
DSP_ASC3:   RET


;/**********************/;
;/* Read Data From ADC */;
;/* LTC1298(12Bit A/D) */;
;/* Output : DPTR      */;
;/**********************/;
; START,SGL,ODD,MSBF = ADC
;   1    1   0   X   = Ch0
;   1    1   1   X   = Ch1
; MSBF = 1 = MSB First
; MSBF = 0 = LSB First
; A = 1101 XXXX = Ch0(DF)
; A = 1111 XXXX = Ch1(FF)
;
READ_ADC:   PUSH    B
            MOV     A,#0DFH             ; Read Ch0
            SETB    SK_ADC              ; Makesure Start Clock
            SETB    DI_ADC              ; Start Bit
            SETB    CS_ADC              ; Makesure CS=High
            CLR     CS_ADC              ; Start Operation
            NOP
            NOP
            NOP
            NOP
            ;
            MOV     B,#4                ; Counter (START/SGL/ODD/MSBF)
READ_AD1:   RLC     A                   ; Rotate Din to Carry
            CLR     SK_ADC              ; Clock Low
            NOP
            NOP
            NOP
            NOP
            MOV     DI_ADC,C            ; Get Data
            SETB    SK_ADC              ; Clock High (ADC Receive Data)
            NOP
            NOP
            NOP
            NOP
            DJNZ    B,READ_AD1          ; Loop Send Data Word 4 Bit
            ;
            SETB    DO_ADC              ; Start for Input Read
            CLR     SK_ADC              ; Clock Low
            NOP
            NOP
            NOP
            NOP
            MOV     B,#5                ; Counter MSB 4-Bit & Null
READ_AD2:   MOV     C,DO_ADC            ; Read Data
            RLC     A                   ; Rotate Carry to Data
            SETB    SK_ADC              ; Clock High
            NOP
            NOP
            NOP
            NOP
            CLR     SK_ADC              ; Clock Low
            NOP
            NOP
            NOP
            NOP
            DJNZ    B,READ_AD2          ; Next bit
            ANL     A,#0FH              ; Only 4bit Data  (D11..D8)
            MOV     DPH,A               ; Save Result MSB Data
            ;
            CLR     A
            MOV     B,#8                ; Counter LSB 8-bit Data
READ_AD3:   MOV     C,DO_ADC            ; Read Data
            RLC     A                   ; Rotate Carry to Data
            SETB    SK_ADC              ; Clock High
            NOP
            NOP
            NOP
            NOP
            CLR     SK_ADC              ; Clock Low
            NOP
            NOP
            NOP
            NOP
            DJNZ    B,READ_AD3          ; Next Bit
            MOV     DPL,A               ; Save Result LSB (D7..D0)
            SETB    CS_ADC              ; Stop Operation
            POP     B
            RET

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PX.0  */;
;/* -> E    = PX.2  */;
;/* -> D4-7 = PX4-7 */;
;/*******************/;
;
INIT_LCD:   MOV     A,#33H              ; DL = 1 ,DL =1
            LCALL   WR_INS
            MOV     A,#32H              ; DL = 1 ,DL =0
            LCALL   WR_INS
            MOV     A,#28H              ; Function set
            LCALL   WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            MOV     A,#0CH              ; Display on/off Control
            LCALL   WR_INS              ; Entry display,cursor off,not blink
            MOV     A,#06H              ; Entry mode set
            LCALL   WR_INS              ; I/D=1 Increment,S=0 cursor shift
            MOV     A,#01H              ; Clear Display
            LCALL   WR_INS              ; Clear Display,Set DD RAM addr=0
            RET

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PX.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD:     PUSH    B
            MOV     B,#50               ; Busy Delay LCD
            DJNZ    B,$
            POP     B
            SETB    E_LCD               ; Enable LCD (E=PX.2=1)
            NOP
            CLR     E_LCD               ; Disable LCD (E=PX.2=0)
            RET

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PUSH    ACC                 ; Save Data Low Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; High Byte & RS=1 Write
            SETB    RS_LCD              ; Set RS For Write Data Select
            LCALL   EN_LCD
            POP     ACC                 ; Restore Data Low Byte
            SWAP    A                   ; Shift Low Byte to High Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; Low Byte & RS=1 Write
            SETB    RS_LCD              ; Set RS For Write Data Select
            LCALL   EN_LCD
            RET

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS:     PUSH    ACC                 ; Save Data Low Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; High Byte & RS=0 Write
            LCALL   EN_LCD
            ;
            POP     ACC                 ; Restore Data Low Byte
            SWAP    A                   ; Shift Low Byte to High Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; Low Byte & RS=0 Write
            LCALL   EN_LCD
            MOV     R2,#2               ; Busy Ins Delay
INS_DLY:    MOV     R3,#250
            DJNZ    R3,$
            DJNZ    R2,INS_DLY
            RET
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD:   SETB    ACC.7
            LCALL   WR_INS
            RET

;/************************************/;
;/*  Print Data data to Serial Port  */;
;/*  to display buffer.              */;
;/*  Usage    : LCALL PRINT_LCD      */;
;/*           : DB   'xxxx',00       */;
;/*  Register : ACC                  */;
;/*  Note     : last byte must be 00 */;
;/************************************/;
;
PRINT_LCD:  POP     DPH
            POP     DPL
PRINT1:     CLR     A
            MOVC    A,@A+DPTR
            CJNE    A,#00H,PRINT2
            SJMP    PRINT3
PRINT2:     LCALL   WR_LCD
            INC     DPTR
            SJMP    PRINT1
PRINT3:     PUSH    DPL
            PUSH    DPH
            RET

            END





            




