;/*********************************/;
;/* Demo Timer 68HC11 For ET-EXP4 */;
;/* Files Name  : LAB11EX4.ASM    */;
;/* CPU Control : 68HC811E2       */;
;/* Assembler   : AS11NEW.EXE     */;
;/*  (Measure IC3 & Display LCD)  */;
;/* Min Frequency = 32 Cycle      */;
;/* Max Frequency = 65536 Cycle   */;
;/*********************************/;
;
;/* Port For Control LCD (Port-B)
PORT_LCD    EQU     $1004               ; Port Data LCD (PortB)
TIC3        EQU     $1014               ; Input Capture #3
TCTL1       EQU     $1020               ; Timer Control Register #1
TCTL2       EQU     $1021               ; Timer Control Register #2
TMSK1       EQU     $1022               ; Timer Mask #1
TFLG1       EQU     $1023               ; Timer Flags #1
TMSK2       EQU     $1024               ; Timer Mask #2
TFLG2       EQU     $1025               ; Timer Flags #2

            ORG     $0000
IC3_STAS    RMB     1                   ; FF= off,0 = 1st,1= 2nd
IC3_RDY     RMB     1                   ; 0 = Not Ready,1=Ready
IC3_BUF1    RMB     2                   ; Rising Edge  (Cycle)
IC3_BUF2    RMB     2                   ; Result Cycle (Rising-Falling)
;
HEX_BUFF    RMB     2
DEC_BUFF    RMB     5

            ORG     $F800               ; Start at Monitor 68HC811E2
START       LDS     #$FF                ; Stack = Last Internal RAM
            LDX     #$FFFF              ; Power-on Delay
PWR_DLY     DEX
            BNE     PWR_DLY
            JSR     INIT_LCD            ; Initial LCD4 Bit (16X2)
            ;
            LDAB    #%00000001          ; Capture IC3 on Rising Edge
            STAB    TCTL2
            LDAB    #%00000001
            STAB    TMSK1               ; Enable IC3 Interrupt
            STAB    TFLG1               ; Clear IC3F
            JSR     IC3_STRT            ; Initial IC3
            JSR     DSP_CYCLE
            CLI                         ; Enable Global Interrupt
            ;
            LDAA    #$00                ; 1st Location Line1
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'DEMO Timer Input'  ; Message on Line-1
            FCB     $00
            LDAA    #$40                ; 1st Location Line2
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'IC3(PA0) =      '  ; Message on Line-2
            FCB     $00
            ;
WAIT_RDY    LDAA    IC3_RDY
            BNE     IC3_DSPY            ; Check IC3 Ready
            BRA     WAIT_RDY
            ;
            ;/* 1 Cycle = 0.5uS
IC3_DSPY    JSR     DSP_CYCLE           ; Display Cycle of Input Signal
            JSR     IC3_STRT
            JMP     WAIT_RDY

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PB0   */;
;/* -> E    = PB2   */;
;/* -> D4-7 = PB4-7 */;
;/*******************/;
;
INIT_LCD    LDAA    #$33                ; DL = 1 ,DL =1
            JSR     WR_INS
            LDAA    #$32                ; DL = 1 ,DL =0
            JSR     WR_INS
            LDAA    #$28                ; Function set
            JSR     WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            LDAA    #$0C                ; Display on/off Control
            JSR     WR_INS              ; Entry display,cursor off,not blink
            LDAA    #$06                ; Entry mode set
            JSR     WR_INS              ; I/D=1 Increment,S=0 cursor shift
            LDAA    #$01                ; Clear Display
            JSR     WR_INS              ; Clear Display,Set DD RAM addr=0
            RTS

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PB.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD      PSHB
            LDAB    #20                 ; 100uS counter
BUSY        DECB
            BNE     BUSY
            PULB
            ORAA    #%00000100          ; Enable LCD (E=PB2=1)
            STAA    PORT_LCD
            ANDA    #%11111011          ; Disable LCD (E=PB2=0)
            STAA    PORT_LCD
            RTS

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; High Byte & RS=1 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; Low Byte & RS=1 Write
            JSR     EN_LCD
            RTS

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS      PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            STAA    PORT_LCD            ; High Byte & RS=0 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            STAA    PORT_LCD            ; Low Byte & RS=0 Write
            JSR     EN_LCD
            PSHX
            LDX     #$0632              ; 5 mSec counter
INS_DLY     DEX
            BNE     INS_DLY
            PULX
            RTS
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD    ORAA    #$80                ; Set ACC.7
            JSR     WR_INS
            RTS

;/********************************/;
;/* Print ASCII data to LCD Port */;
;/* Usage    : JSR  PRINT_LCD    */;
;/*          : FCC  'xxxxxxxx'   */;
;/*          : FCB  $00          */;
;/* Note     : last byte = $00   */;
;/********************************/;
;
PRINT_LCD   PULX                        ; Pop Stack
            PSHA
            PSHB
PRINT1      LDAA    0,X                 ; Get Data For Print
            CMPA    #$00                ; Check Last Byte
            BNE     PRINT2              ; Jump if not Last Byte
            BRA     PRINT3
PRINT2      JSR     WR_LCD              ; Write Data to LCD
            INX
            BRA     PRINT1
PRINT3      INX
            PULB
            PULA
            PSHX                        ; Push Stack
            RTS

;/*********************/;
;/* Display Cycle IC3 */;
;/*********************/;
;
DSP_CYCLE   LDAA    #$4B                ; Cursor Display LCD
            JSR     GOTO_LCD
            ;
            LDD     IC3_BUF2
            CPD     #32
            BHI     DSP_IC3X
            JSR     PRINT_LCD
            FCC     'Over!'
            FCB     $00
            BRA     DSP_UNT3
DSP_IC3X    STD     HEX_BUFF
            JSR     HTOD                    ; Convert to 5 Decimal
            LDAA    DEC_BUFF+0
            JSR     WR_LCD
            LDAA    DEC_BUFF+1
            JSR     WR_LCD
            LDAA    DEC_BUFF+2
            JSR     WR_LCD
            LDAA    DEC_BUFF+3
            JSR     WR_LCD
            LDAA    DEC_BUFF+4
            JSR     WR_LCD
DSP_UNT3    RTS

;/**************************/;
;/* Convert HEX to Decimal */;
;/* Input  : HEX_BUFF      */;
;/* Output : DEC_BUFF      */;
;/**************************/;
;
HTOD        PSHX                            ; Save-IX
            LDD     HEX_BUFF
            LDX     #10000
            IDIV
            XGDX
            ADDB    #$30
            STAB    DEC_BUFF+0
            XGDX
            LDX     #1000
            IDIV
            XGDX
            ADDB    #$30
            STAB    DEC_BUFF+1
            XGDX
            LDX     #100
            IDIV
            XGDX
            ADDB    #$30
            STAB    DEC_BUFF+2
            XGDX
            LDX     #10
            IDIV
            ADDB    #$30
            STAB    DEC_BUFF+4
            XGDX
            ADDB    #$30
            STAB    DEC_BUFF+3
            PULX                            ; Restore-IX
            RTS

;/*************************/;
;/* Initial IC3 Parameter */;
;/*************************/;
;
IC3_STRT    LDAB    #$FF
            STAB    IC3_STAS                ; FF= off,0 = 1st,1= 2nd
            LDD     #0
            STD     IC3_BUF2
            STAB    IC3_RDY
            LDAB    #%00000001
            STAB    TFLG1                   ; Clear IC3F
            LDAB    TMSK1
            ORAB    #%00000001
            STAB    TMSK1                   ; Enable IC3X
            RTS

;/***************************/;
;/* IC3 Measure Pulse Width */;
;/*    (Rising...Rising)    */;
;/* Max Cycle = 65535 Cycle */;
;/***************************/;
;
IC3_INT     INC     IC3_STAS
            LDAB    IC3_STAS
            BNE     IC3_EDG2
IC3_EDG1    LDD     TIC3                    ; Read 1st Rising Edge
            STD     IC3_BUF1
            BRA     IC3_OUT
IC3_EDG2    LDD     TIC3                    ; Read 2nd Rising Edge
            SUBD    IC3_BUF1                ; 2nd Rising - 1st Rising
            STD     IC3_BUF2                ; Result of Cycle
            INC     IC3_RDY
            LDAB    TMSK1
            ANDB    #%11111110
            STAB    TMSK1                   ; Disable IC3X
IC3_OUT     LDAB    #%00000001
            STAB    TFLG1                   ; Clear IC3F
            RTI

            ORG     $FFEA
            FDB     IC3_INT                 ; IC3 Vector
            ORG     $FFFE                   ; Reset Vector
            FDB     START

            END






;/**************************************/;
;/* Multiply 2Byte^X x ACCA = RESULT   */;
;/* Function : ^IX+1,2 x ACCA = Result */;
;/* Input    : Reg-X = Point to 2Byte  */;
;/*          : ACCA  = Multiplyer      */;
;/* Output   : Buffer RESULT (0..2)    */;
;/**************************************/;
;
MUL_2HEX    PSHA                            ; Save hex number on the stack
            LDAB    1,X                     ; Get low byte of period
            MUL                             ; multiply
            STAB    RESULT+2                ; Lowest byte of total result
            PULB                            ; Get copy of hex rep of %
            PSHA                            ; To be added to next mul result
            LDAA    0,X                     ; Get high byte of period
            MUL                             ; Multiply
            STD     RESULT                  ; Store this result in RESULT
            PULB                            ; Part to be added to last mul
            CLRA                            ; Hi byte of accd = 0
            ADDD    RESULT                  ; Add accd to last mul result
            STD     RESULT                  ; 24 bit result Save in RESULT (0..2)
            RTS

;/*******************************/;
;/* Divide RESULT By X => ACCD  */;
;/* Function : RESULT/X = D     */;
;/* Input    : RESULT           */;
;/*          : Reg-X            */;
;/* Output   : ACCD (A B : -D-) */;
;/*******************************/;
;
DIV_3HEX    LDD     RESULT                  ; Get Upper 16 bits of dividend
            LDX     #$0064                  ; Load X with divisor (100%)
            IDIV                            ; Divide  D/IX => IX ; r => D
            PSHB                            ; Save Remainder
            XGDX                            ; ACCB = Result of 1st Divide
            PULA                            ; Get Remainder to ACCA
            PSHB                            ;*/ Save hi byte of result /*;
            LDAB    RESULT+2                ; Get Lower 8 Bits of dividend
            LDX     #$0064                  ; Load X with divisor (100%)
            IDIV                            ; Divide D/IX => IX ; r => D
            XGDX                            ; ACCB = Result (Low-Byte)
            PULA                            ; ACCA = Result (High-Byte)
            RTS

;/********************************/;
;/* OC1 Interupt Service Routine */;
;/*       (Low Pulse of PWM)     */;
;/********************************/;
;
OC1_INTR    LDX     #REGBAS                 ; Base Register Addressing
            LDD     TOC1,X                  ; Get old OC1 time
            ADDD    PERIOD                  ; Add one period
            STD     TOC1,X                  ; Store next OC1 time
            LDAA    #%10000000              ; Clear OC1 Interupt Flag
            STAA    TFLG1,X
            RTI

;/********************************/;
;/* OC2 Interupt Service Routine */;
;/*     (High Pulse of PWM)      */;
;/********************************/;
;
OC2_INTR    LDX     #REGBAS                 ; Base Register Addressing
            LDAA    NEW_PWM                 ; Check Update Status
            BNE     NEW_TIME                ; NEW_PWM <> 0 is Update
            ;
            LDD     TOC2,X                  ; Get old OC2 Time
            ADDD    PERIOD                  ; TOC1 + Period = ACCD
            STD     TOC2,X                  ; Update New OC2 Time
            LDAA    #%01000000              ; Clear OC2 Interupt Flag
            STAA    TFLG1,X
            RTI
            ;
NEW_TIME    LDD     TOC1,X                  ; Use Time OC1 Reference
            ADDD    PERIOD                  ; TOC1 + Period = ACCD (50%-LOW)
            ADDD    PERIOD                  ; Add a second period  (50%-HIGH)
            SUBD    PWIDTH                  ; 100%PWM - PWIDTH = PWM(HIGH)
            STD     TOC2,X                  ; Store Next OC2 Time
            CLR     NEW_PWM                 ; Clear Update PWM Status
            LDAA    #%01000000              ; Clear OC2 Interupt Flag
            STAA    TFLG1,X                 
            RTI

            ORG     $FFE6
            FDB     OC2_INTR                ; OC2 Interupt Service
            ;
            ORG     $FFE8
            FDB     OC1_INTR                ; OC1 Onterupt Service
            ;
            ORG     $FFFE
            FDB     MAIN                    ; Reset Vector

            END





