;/********************************/;
;/* Demo EEPROM 9346 For ET-EXP4 */;
;/* Files Name  : LAB7-EX2.ASM   */;
;/* CPU Control : 68HC811E2      */;
;/* Assembler   : AS11NEW.EXE    */;
;/*    (Write & Read to LCD)     */;
;/********************************/;
;
;/* Interface SPI With 93C46
;/* CPU MISO (PD2) -- EEP DOUT (Pin 4)
;/* CPU MOSI (PD3) -- EEP DIN  (Pin 3)
;/* CPU SCK  (PD4) -- EEP CLK  (Pin 2)
;/* CPU SS#  (PD0) -- EEP CS   (Pin 1)
;
PORTD       EQU     $1008               ; PortD Data Register
DDRD        EQU     $1009               ; PortD Data Direction
SPCR        EQU     $1028               ; SPI Control Register
SPSR        EQU     $1029               ; SPI Status Register
SPDR        EQU     $102A               ; SPI Data Register
CS_EEP      EQU     #%00000001          ; CS EEP = PD0
CS_ADC      EQU     #%00000010          ; CS ADC = PD1
;
;/* Port For Control LCD (Port-B)
PORT_LCD    EQU     $1004               ; Port Data LCD (PortB)

            ORG     $00                 ; Internal RAM
CNT_WRITE   RMB     1

            ORG     $F800               ; Start Monitor of 68H1811E2
START       LDS     #$FF                ; Stack on Last Internal RAM
            LDAA    #%00111011          ; PD2=In:PD0,1,3,4,5=Out
            STAA    DDRD
            LDX     #$1000              ; X = Base Register
            BCLR    $08,X CS_EEP        ; PD0 = 0 = Stop EEP
            BSET    $08,X CS_ADC        ; PD1 = 1 = Stop ADC
            LDAA    #%01010000          ; SPI=Master,SCK Normal=0
            STAA    SPCR                ; Clock = E/32,Active Rising
            JSR     READ_EEP
            ;
            LDX     #$FFFF              ; Power-on Delay
PWR_DLY     DEX
            BNE     PWR_DLY
            JSR     INIT_LCD            ; Initial LCD4 Bit (16X2)
            LDAA    #$00                ; 1st Location Line1
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'DEMO EEPROM 9346'  ; Message on Line-1
            FCB     $00
            LDAA    #$40                ; 1st Location Line2
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'Read0-7:        '  ; Message on Line-2
            FCB     $00
            ;
            JSR     EWEN_EEP            ; Enable Erase/Write
            LDY     #TAB_ADDR           ; Address
            LDX     #TAB_DATA           ; Data
            LDAB    #8
            STAB    CNT_WRITE
            ;
WRITE_DAT   LDAA    0,Y                 ; Get Address
            LDAB    0,X                 ; Get Data
            JSR     WRITE_EEP
            INX                         ; Next Data
            INY
            DEC     CNT_WRITE
            BNE     WRITE_DAT
            JSR     EWDS_EEP            ; Disable Erase/Write
            ;
            LDAA    #$48                ; Display EEPROM
            JSR     GOTO_LCD
            LDY     #8                  ; 8 Byte Read
            LDX     #TAB_ADDR
READ_DAT    LDAA    0,X                 ; Get Address
            JSR     READ_EEP
            JSR     WR_LCD              ; Display Read
            INX                         ; Next Read
            DEY
            BNE     READ_DAT

LOOP        BRA     *

TAB_ADDR    FCB     $00,$01,$02,$03
            FCB     $04,$05,$06,$07
TAB_DATA    FCC     'ASDFGHJK'

;/**************************/;
;/* Interface SPI to 93C46 */;
;/* - EWEN Command 8-Bit   */;
;/* Format : 10 & 011XXXXX */;
;/*  (Eeprom Write Enable) */;
;/**************************/;
;
EWEN_EEP    PSHX                        ; Save Register
            PSHA
            LDX     #$1000              ; X = Base Register
            BSET    $08,X CS_EEP        ; CS = 1 = Active
            LDAA    #%00000010          ; XXXXXX10
            STAA    SPDR                ; Send SPI Byte1
EWEN_EE1    LDAA    SPSR                ; Read Status
            BPL     EWEN_EE1
            LDAA    #%01100000          ; 011XXXXX
            STAA    SPDR                ; Send Command EWEN
EWEN_EE2    LDAA    SPSR                ; Read Status
            BPL     EWEN_EE2
            BCLR    $08,X CS_EEP        ; CS = 0 = Stop
            PULA                        ; Restore Register
            PULX
            RTS

;/**************************/;
;/* Interface SPI to 93C46 */;
;/* - EWDS Command 8-Bit   */;
;/* Format : 10 & 000XXXXX */;
;/* (Eeprom Write Disable) */;
;/**************************/;
;
EWDS_EEP    PSHX                        ; Save Register
            PSHA
            LDX     #$1000              ; X = Base Register
            BSET    $08,X CS_EEP        ; CS = 1 = Active
            LDAA    #%00000010          ; XXXXXX10
            STAA    SPDR                ; Send SPI Byte1
EWDS_EE1    LDAA    SPSR                ; Read Status
            BPL     EWDS_EE1
            LDAA    #%00000000          ; 000XXXXX
            STAA    SPDR                ; Send Command EWEN
EWDS_EE2    LDAA    SPSR                ; Read Status
            BPL     EWDS_EE2
            BCLR    $08,X CS_EEP        ; CS = 0 = Stop
            PULA                        ; Restore Register
            PULX
            RTS

;/**************************/;
;/* Interface SPI to 93C46 */;
;/* - ERASE Command 8-Bit  */;
;/* Input  : ACCA(Addr)    */;
;/* Format : 11 & 1,A6..A0 */;
;/*   (Erase By Address)   */;
;/**************************/;
;
ERASE_EEP   PSHX                        ; Save X
            PSHA                        ; Save Address
            LDX     #$1000              ; X = Base Register
            BSET    $08,X CS_EEP        ; CS = 1 = Active
            LDAA    #%00000011          ; XXXXXX11
            STAA    SPDR                ; Send SPI Byte1
ERASE_EE1   LDAA    SPSR                ; Read Status
            BPL     ERASE_EE1
            PULA                        ; Get Address
            ORAA    #%10000000          ; 1,Addr(A6..A0)
            STAA    SPDR                ; Send Command & Address
ERASE_EE2   LDAA    SPSR                ; Read Status
            BPL     ERASE_EE2
            BCLR    $08,X CS_EEP        ; CS = 0 = Stop
            ;
            BSET    $08,X CS_EEP        ; CS = 1 = Active Check Status
ERASE_EE3   LDAA    #%00000000          ; XXXXXXXX
            STAA    SPDR                ; Send SPI Dummy Byte
ERASE_EE4   LDAA    SPSR                ; Read Status
            BPL     ERASE_EE4
            LDAA    SPDR
            BEQ     ERASE_EE3           ; Repeat Until Ready
            BCLR    $08,X CS_EEP        ; CS = 0 = Stop
            PULX                        ; Restore X
            RTS

;/**************************/;
;/* Interface SPI to 93C46 */;
;/* - ERAL Command 8-Bit   */;
;/* Format : 10 & 010XXXXX */;
;/*   (Erase All Address)  */;
;/**************************/;
;
ERAL_EEP    PSHX                        ; Save Register
            PSHA
            LDX     #$1000              ; X = Base Register
            BSET    $08,X CS_EEP        ; CS = 1 = Active
            LDAA    #%00000010          ; XXXXXX10
            STAA    SPDR                ; Send SPI Byte1
ERAL_EE1    LDAA    SPSR                ; Read Status
            BPL     ERAL_EE1
            LDAA    #%01000000          ; 010XXXXX
            STAA    SPDR                ; Send Command EWEN
ERAL_EE2    LDAA    SPSR                ; Read Status
            BPL     ERAL_EE2
            BCLR    $08,X CS_EEP        ; CS = 0 = Stop
            ;
            BSET    $08,X CS_EEP        ; CS = 1 = Active Check Status
ERAL_EE3    LDAA    #%00000000          ; XXXXXXXX
            STAA    SPDR                ; Send SPI Dummy Byte
ERAL_EE4    LDAA    SPSR                ; Read Status
            BPL     ERAL_EE4
            LDAA    SPDR
            BEQ     ERAL_EE3            ; Repeat Until Ready
            BCLR    $08,X CS_EEP        ; CS = 0 = Stop
            PULA                        ; Restore Register
            PULX
            RTS

;/**************************/;
;/* Interface SPI to 93C46 */;
;/* - WRAL Command 8-Bit   */;
;/* Input  : ACCA(Data)    */;
;/* Format : 10 & 001XXXXX */;
;/*  (Write Data All Addr) */;
;/**************************/;
;
WRAL_EEP    PSHX                        ; Save X
            PSHA                        ; Save Data
            LDX     #$1000              ; X = Base Register
            BSET    $08,X CS_EEP        ; CS = 1 = Active
            LDAA    #%00000010          ; XXXXXX10
            STAA    SPDR                ; Send SPI Byte1
WRAL_EE1    LDAA    SPSR                ; Read Status
            BPL     WRAL_EE1
            LDAA    #%00100000          ; 001XXXXX
            STAA    SPDR                ; Send Command
WRAL_EE2    LDAA    SPSR                ; Read Status
            BPL     WRAL_EE2
            PULA                        ; Get Data
            STAA    SPDR                ; Send SPI Byte3
WRAL_EE3    LDAA    SPSR
            BPL     WRAL_EE3
            BCLR    $08,X CS_EEP        ; CS = 0 = Stop
            ;
            BSET    $08,X CS_EEP        ; CS = 1 = Active Check Status
WRAL_EE4    LDAA    #%00000000          ; XXXXXXXX
            STAA    SPDR                ; Send SPI Dummy Byte
WRAL_EE5    LDAA    SPSR                ; Read Status
            BPL     WRAL_EE5
            LDAA    SPDR
            BEQ     ERAL_EE4            ; Repeat Until Ready
            BCLR    $08,X CS_EEP        ; CS = 0 = Stop
            PULX                        ; Restore X
            RTS

;/**************************/;
;/* Interface SPI to 93C46 */;
;/* - WRITE Command 8-Bit  */;
;/* Input  : ACCA(Addr)    */;
;/*        : ACCB(Data)    */;
;/* Format : 10 & 1,A6..A0 */;
;/*  (Write Data By Addr)  */;
;/**************************/;
;
WRITE_EEP   PSHX                        ; Save X
            PSHA                        ; Save Address
            LDX     #$1000              ; X = Base Register
            BSET    $08,X CS_EEP        ; CS = 1 = Active
            LDAA    #%00000010          ; XXXXXX10
            STAA    SPDR                ; Send SPI Byte1
WRITE_EE1   LDAA    SPSR                ; Read Status
            BPL     WRITE_EE1
            PULA                        ; Get Address
            ORAA    #%10000000          ; 1,Addr(A6..A0)
            STAA    SPDR                ; Send Command
WRITE_EE2   LDAA    SPSR                ; Read Status
            BPL     WRITE_EE2
            TBA                         ; Get Data
            STAA    SPDR                ; Send SPI Byte3
WRITE_EE3   LDAA    SPSR
            BPL     WRITE_EE3
            BCLR    $08,X CS_EEP        ; CS = 0 = Stop
            ;
            BSET    $08,X CS_EEP        ; CS = 1 = Active Check Status
WRITE_EE4   LDAA    #%00000000          ; XXXXXXXX
            STAA    SPDR                ; Send SPI Dummy Byte
WRITE_EE5   LDAA    SPSR                ; Read Status
            BPL     WRITE_EE5
            LDAA    SPDR
            BEQ     WRITE_EE4           ; Repeat Until Ready
            BCLR    $08,X CS_EEP        ; CS = 0 = Stop
            PULX                        ; Restore X
            RTS

;/**************************/;
;/* Interface SPI to 93C46 */;
;/* - READ Command 8-Bit   */;
;/* Input  : ACCA(Addr)    */;
;/* Output : ACCA(Data)    */;
;/* Format : 11 & 0,A6..A0 */;
;/*   (Read Data 1-Byte)   */;
;/**************************/;
;
READ_EEP    PSHX                        ; Save X
            PSHA                        ; Save Address
            LDX     #$1000              ; X = Base Register
            BSET    $08,X CS_EEP        ; CS = 1 = Active
            LDAA    #%00000110          ; XXXXX110
            STAA    SPDR                ; Send SPI Byte1
READ_EE1    LDAA    SPSR                ; Read Status
            BPL     READ_EE1
            PULA                        ; Get Address
            ASLA                        ; Shift MSB <-- A6..A0
            STAA    SPDR                ; Address A6..A0,X
READ_EE2    LDAA    SPSR                ; Read Status
            BPL     READ_EE2
            ;
            LDAA    #%00000000          ; Dummy Byte to Read Data
            STAA    SPDR                ; Send SPI Byte3
READ_EE3    LDAA    SPSR
            BPL     READ_EE3
            LDAA    SPDR                ; Read Data Result (D7-D0)
            BCLR    $08,X CS_EEP        ; CS = 0 = Stop
            PULX                        ; Restore X
            RTS

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PB0   */;
;/* -> E    = PB2   */;
;/* -> D4-7 = PB4-7 */;
;/*******************/;
;
INIT_LCD    LDAA    #$33                ; DL = 1 ,DL =1
            JSR     WR_INS
            LDAA    #$32                ; DL = 1 ,DL =0
            JSR     WR_INS
            LDAA    #$28                ; Function set
            JSR     WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            LDAA    #$0C                ; Display on/off Control
            JSR     WR_INS              ; Entry display,cursor off,not blink
            LDAA    #$06                ; Entry mode set
            JSR     WR_INS              ; I/D=1 Increment,S=0 cursor shift
            LDAA    #$01                ; Clear Display
            JSR     WR_INS              ; Clear Display,Set DD RAM addr=0
            RTS

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PB.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD      PSHB
            LDAB    #20                 ; 100uS counter
BUSY        DECB
            BNE     BUSY
            PULB
            ORAA    #%00000100          ; Enable LCD (E=PB2=1)
            STAA    PORT_LCD
            ANDA    #%11111011          ; Disable LCD (E=PB2=0)
            STAA    PORT_LCD
            RTS

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; High Byte & RS=1 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; Low Byte & RS=1 Write
            JSR     EN_LCD
            RTS

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS      PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            STAA    PORT_LCD            ; High Byte & RS=0 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            STAA    PORT_LCD            ; Low Byte & RS=0 Write
            JSR     EN_LCD
            PSHX
            LDX     #$0632              ; 5 mSec counter
INS_DLY     DEX
            BNE     INS_DLY
            PULX
            RTS
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD    ORAA    #$80                ; Set ACC.7
            JSR     WR_INS
            RTS

;/********************************/;
;/* Print ASCII data to LCD Port */;
;/* Usage    : JSR  PRINT_LCD    */;
;/*          : FCC  'xxxxxxxx'   */;
;/*          : FCB  $00          */;
;/* Note     : last byte = $00   */;
;/********************************/;
;
PRINT_LCD   PULX                        ; Pop Stack
            PSHA
            PSHB
PRINT1      LDAA    0,X                 ; Get Data For Print
            CMPA    #$00                ; Check Last Byte
            BNE     PRINT2              ; Jump if not Last Byte
            BRA     PRINT3
PRINT2      JSR     WR_LCD              ; Write Data to LCD
            INX
            BRA     PRINT1
PRINT3      INX
            PULB
            PULA
            PSHX                        ; Push Stack
            RTS

            ORG     $FFFE               ; Reset Vector
            FDB     START

            END

